<?php

namespace Drupal\iconify_icons;

use Drupal\Core\File\FileSystemInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Psr\Log\LoggerInterface;

/**
 * Service for Iconify functionality.
 */
class IconifyService implements IconifyServiceInterface {

  public const SEARCH_API_ENDPOINT = 'https://api.iconify.design/search';
  public const COLLECTIONS_API_ENDPOINT = 'https://api.iconify.design/collections';

  // Arguments [$collection, $icon_name].
  public const DESIGN_DOWNLOAD_API_ENDPOINT = 'https://api.iconify.design/%s/%s.svg';

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The Logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The fileSystem service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * IconifyService constructor.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Psr\Log\LoggerInterface $logger
   *   The Logger service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The FileSystem service.
   */
  public function __construct(ClientInterface $http_client, LoggerInterface $logger, FileSystemInterface $fileSystem) {
    $this->httpClient = $http_client;
    $this->logger = $logger;
    $this->fileSystem = $fileSystem;
  }

  /**
   * {@inheritdoc}
   */
  public function getIcons(string $query, string $collection, int $limit = 10): array {
    try {
      $response = $this->httpClient->request('GET', $this::SEARCH_API_ENDPOINT, [
        'query' => [
          'query' => urlencode($query),
          'prefixes' => $collection,
        ],
      ]);

      $data = json_decode($response->getBody()->getContents(), TRUE, 512, JSON_THROW_ON_ERROR);
      return $data['icons'] ?? [];
    }
    catch (RequestException $e) {
      // Handle request exception (e.g., log error, return empty array)
      $this->logger->error('JSON decode error: {message}', ['message' => $e->getMessage()]);
      return [];
    }
    catch (\JsonException $e) {
      $this->logger->error('JSON decode error: {message}', ['message' => $e->getMessage()]);
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCollections(): array {
    try {
      $response = $this->httpClient->request('GET', $this::COLLECTIONS_API_ENDPOINT, []);
      return json_decode($response->getBody()->getContents(), TRUE, 512, JSON_THROW_ON_ERROR);
    }
    catch (RequestException $e) {
      // Handle request exception (e.g., log error, return empty array)
      $this->logger->error('Error fetching collections: ' . $e->getMessage());
      return [];
    }
    catch (\JsonException $e) {
      $this->logger->error('JSON decode error: {message}', ['message' => $e->getMessage()]);
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getIconsByCollection(string $collection): array {
    try {
      $response = $this->httpClient->request('GET', $this::COLLECTIONS_API_ENDPOINT, [
        'query' => [
          'prefixes' => $collection,
        ],
      ]);

      $data = json_decode($response->getBody(), TRUE, 512, JSON_THROW_ON_ERROR);
      return $data['icons'];
    }
    catch (RequestException $e) {
      // Handle request exception (e.g., log error, return empty array)
      $this->logger->error('Error fetching icons: ' . $e->getMessage());
      return [];
    }
  }

  /**
   * Calculates a file system path to cache an icon based on its parameters.
   *
   * @param string $collection
   *   The collection.
   * @param string $icon_name
   *   The icon name.
   * @param array $query_options
   *   Query options.
   *
   * @return string
   *   The Path in drupal file system.
   */
  protected function iconSettingsToPath(string $collection, string $icon_name, array $query_options) {
    return 'public://iconify-icons/' . implode('/', [
      $collection,
      $icon_name,
      $query_options['width'],
      $query_options['height'],
      $query_options['download'],
      $query_options['box'],
      $query_options['color'],
      $query_options['flip'],
      $query_options['rotate'],
    ]);
  }

  /**
   * Gets the icon from cache and return empty string if it doesn't exist.
   *
   * @param string $collection
   *   The collection.
   * @param string $icon_name
   *   The icon name.
   * @param array $query_options
   *   Query options.
   *
   * @return string
   *   The svg or '' if it doesn't exist yet.
   */
  protected function getIconFromCache(string $collection, string $icon_name, array $query_options): string {
    // Translate settings to path.
    $path = $this->iconSettingsToPath($collection, $icon_name, $query_options);

    if ($this->fileSystem->realpath($path . '/' . $icon_name . '.svg')) {
      return file_get_contents($path . '/' . $icon_name . '.svg');
    }
    return '';
  }

  /**
   * Qualifies the default parameters and sets if any are missing.
   *
   * @param array $parameters
   *   The parameters to qualify.
   *
   * @return array
   *   The qualified parameters.
   */
  protected function setDefaultParameters(array $parameters): array {
    return [
      'width' => $parameters['width'] ?? 25,
      'height' => $parameters['height'] ?? 25,
      'color' => $parameters['color'] ?? '#000',
      'flip' => $parameters['flip'] ?? '',
      'rotate' => $parameters['rotate'] ?? '',
      'download' => $parameters['download'] ?? FALSE,
      'box' => $parameters['box'] ?? FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function generateSvg(string $collection, string $icon_name, array $parameters = []): string {
    $parameters = $this->setDefaultParameters($parameters);
    if ($icon = $this->getIconFromCache($collection, $icon_name, $parameters)) {
      return $icon;
    }
    try {
      $response = $this->httpClient->request('GET', sprintf($this::DESIGN_DOWNLOAD_API_ENDPOINT, $collection, $icon_name), [
        'query' => $parameters,
      ]);

      $directory = $this->iconSettingsToPath($collection, $icon_name, $parameters);
      $icon = $response->getBody()->getContents();

      if ($this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY)) {
        $filepath = $directory . '/' . $icon_name . '.svg';
        $this->fileSystem->saveData($icon, $filepath, FileSystemInterface::EXISTS_REPLACE);
      }
      return $icon;
    }
    catch (RequestException $e) {
      // Handle request exception (e.g., log error, return empty array)
      $this->logger->error('Error generating svg icon: ' . $e->getMessage());
      return '';
    }
    catch (\JsonException $e) {
      $this->logger->error('JSON decode error: {message}', ['message' => $e->getMessage()]);
      return '';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getIconSource(string $collection, string $icon_name, array $parameters = []): string {
    $width = $parameters['width'] ?? '';
    $height = $parameters['height'] ?? '';
    $color = $parameters['color'] ?? '';
    $flip = $parameters['flip'] ?? '';
    $rotate = $parameters['rotate'] ?? '';

    return sprintf(
      "https://api.iconify.design/%s/%s.svg?width=%s&height=%s&color=%s&flip=%s&rotate=%s",
      $collection,
      $icon_name,
      $width,
      $height,
      $color,
      $flip,
      $rotate
    );
  }

}
